#include "sys.h"

volatile uint8_t h, m, s, ms;
volatile uint8_t blink;
volatile uint8_t status;
volatile uint8_t mode;
volatile uint8_t al_stat;
volatile uint8_t al_day[4];
volatile uint8_t al_hour[4];
volatile uint8_t al_min[4];
volatile uint8_t cd_h, cd_m, cd_s;
volatile uint8_t cd_en, cd_ms;
volatile uint8_t al_timeout;
volatile uint8_t al_def_time;
volatile uint8_t beep_ms;
volatile uint8_t dcntr;
volatile uint8_t blink_ms;
volatile uint8_t blink_max;

volatile uint8_t esp_timeout;
volatile uint8_t esp_llen;
volatile uint8_t al_def_time = AL_TIME;	//specifies the beeping time (sec)

char str_ssid[32] EEMEM = "noAP";
char str_wifipass[32] EEMEM = "noPWD";

const uint8_t bootpass[6] PROGMEM = {2,7,1,8,2,8};

const char html_begin[] PROGMEM = "<html><head><title>ALARMCLOCK</title></head><body bgcolor=#646482><center><h1>ALARMCLOCK v1.1</h1><h1>configuration</h1><form method=\"post\"><table style=\"color:white; background-color:#323230; margin:5px; padding:5px;\"><tr style=\"background-color:#d24230;\"><td>WHAT</td><td>VALUE</td></tr>";
const char html_end[] PROGMEM = "<tr><td colspan=\"2\"><center><input type=\"submit\" value=\"set values\"></center></td></tr></table></form></center></body></html>";
const char str_get[4] PROGMEM = "GET";
const char str_post[5] PROGMEM = "POST";
const char str_el[5] PROGMEM = "\r\n\r\n";

volatile uint8_t st_mach;
volatile uint8_t cnt_get;
volatile uint8_t cnt_post;
volatile uint8_t cnt_el;

const char character[23] PROGMEM =
{
//B C D E G F DP A
0b11110101,	//0
0b11000000,	//1
0b10111001,	//2
0b11101001,	//3
0b11001100,	//4
0b01101101,	//5
0b01111101,	//6
0b11000001,	//7
0b11111101,	//8
0b11101101,	//9
0b11011101,	//a
0b01111100,	//b
0b00111000,	//c
0b11111000,	//d
0b10111101,	//e
0b00011101,	//f
0b01111000,	//o-16
0b00111100,	//t-17
0b01011000,	//n-18
0b10011101,	//p-19
0b01011100, //h-20
0b00011000,	//r-21
0b00110100	//L-22
};

void alarm_start()
{
	set_stat(ST_ALARM);
	beep_ms=0;
	al_timeout = al_def_time;
}

//bootloader check
uint8_t bootloader_pwd()
{
	//disp and inputs should be initialized first
	//we need TC1 for blink too
	uint8_t pwd[6] = {0,0,0,0,0,0};
	uint8_t dig = 0;	//digit
	uint8_t dir = 0;	//direction, 1cw, 2ccw
	uint8_t e =0;
	uint8_t ok;

	set_stat(ST_REFRESH);

	while (1)
	{
		//check inputs
		e = event_is(EVENT_RCCW | EVENT_RCW);
		if (e)
		{
			//clear events
			event_clr(EVENT_RCW | EVENT_RCCW);

			//if first move
			if (!dir) dir = e;

			else if(dir != e)
			{
				dir = e;
				dig = (dig+1)%6;
			}

			else pwd[dig] = (pwd[dig]+1)%10;
		}

		//set display
		if (is_stat(ST_REFRESH))
		{
			disp_buf[0] = pgm_read_byte( character + pwd[0]) | ((blink&&(dig==0))?DSP_DP:0);
			disp_buf[1] = pgm_read_byte( character + pwd[1]) | ((blink&&(dig==1))?DSP_DP:0);
			disp_buf[2] = pgm_read_byte( character + pwd[2]) | ((blink&&(dig==2))?DSP_DP:0);
			disp_buf[3] = pgm_read_byte( character + pwd[3]) | ((blink&&(dig==3))?DSP_DP:0);
			disp_buf[4] = pgm_read_byte( character + pwd[4]) | ((blink&&(dig==4))?DSP_DP:0);
			disp_buf[5] = pgm_read_byte( character + pwd[5]) | ((blink&&(dig==5))?DSP_DP:0);
			disp_show();
			clear_stat(ST_REFRESH);
		}

		//check password
		if (event_is(EVENT_BTN1))
		{
			event_clr(EVENT_BTN1);
			ok=1;
			for (uint8_t i = 0; i<6; i++) if (pwd[i] != pgm_read_byte(bootpass + i)) ok = 0;

			//if ok, then return 1;
			if (ok) return 1;
			return 0;
		}
	}

	return 0;
}

uint8_t bootloader_check()
{
	if (btn_is(ROT_C)) return 0;
	return 1;
}

void esp_conf(uint8_t _con)
{
	//modul jraindtsa
	if (!(esp_stat&0x80) && _con)
	{
		esp_on(ESP_MODE_N);
		esp_ack_wait(ESP_READY, 50);
	}

	if ((esp_stat&0x80) && !_con)
	{
		esp_off();
		return;
	}

	switch(_con)
	{
	case ESP_CON_AP:
		{
			uart_send("AT+CWMODE=2\r\n");
			esp_ack_wait(ESP_OK, 25);

			uart_send("AT+CIPMUX=1\r\n");
			esp_ack_wait(ESP_OK, 25);

			uart_send("AT+CWSAP=\"ALARMCLOCK\",\"\",9,0\r\n");
			esp_ack_wait(ESP_OK, 50);

			uart_send("AT+CIPSERVER=1,80\r\n");
			esp_ack_wait(ESP_OK, 50);
		}; break;
	case ESP_CON_CL:
		{
			uart_send("AT+CWMODE=1\r\n");
			esp_ack_wait(ESP_OK, 25);

			uart_send("AT+CIPMUX=1\r\n");
			esp_ack_wait(ESP_OK, 25);

			//send authentication data and connect
			uart_send("AT+CWJAP=\"");
			uart_send_eeprom(str_ssid);
			uart_send("\",\"");
			uart_send_eeprom(str_wifipass);
			uart_send("\r\n");

			esp_ack_wait(ESP_OK, 200);

			uart_send("AT+CIPSERVER=1,47\r\n");
			esp_ack_wait(ESP_OK, 50);
		}; break;
	}
}

uint8_t esp_ack_wait(uint8_t type, uint8_t tout)
{
	esp_timeout = tout;
	esp_stat &= 0xf0;
	
	while(esp_timeout) if ((esp_stat & 0x0f) == type) return 1;

	return 0;
}

void esp_read_ack()
{
	uint8_t st;
	uint8_t min = esp_llen<5?esp_llen:5;

	if (min) st = 0x0f;
	else st = 0;

	//rtelmezs kezds
	for (uint8_t i = 0; i<min; i++)
	{
		if (pgm_read_byte(i+str_ready) != esp_buffer[i]) st &= ~(1<<0);
		if (pgm_read_byte(i+str_ok) != esp_buffer[i]) st &= ~(1<<1);
		if (pgm_read_byte(i+str_error) != esp_buffer[i]) st &= ~(1<<2);
		if (pgm_read_byte(i+str_pack) != esp_buffer[i]) st &= ~(1<<3);
	}

	esp_stat = (esp_stat&0xf0) | st;
}

void init_vars()
{
	h = 0;
	m = 0;
	s = 0;
	ms = 0;
	blink = 0;
	status = 0;
	mode = 0;
	al_stat = 0;
	((uint32_t*)al_day)[0] = 0;
	((uint32_t*)al_hour)[0] = 0;
	((uint32_t*)al_min)[0] = 0;
	cd_h=0;
	cd_m=0;
	cd_s=0;
	cd_en=0;
	cd_ms=0;
	al_timeout;
	beep_ms = 0;
	dcntr = 0;
	blink_ms = 0;
	blink_max = BLINK_TIME;
	esp_timeout=0;
	esp_llen =0;
	al_def_time = AL_TIME;
}

//ISR
//id lptetse, 10ms-onknt hvdik meg
//egyb idvel kapcsolatos vltozlk lptetse
ISR(TIMER1_COMPA_vect)
{
	//id lptetse
	ms++;
	if (ms==100) {ms=0; s++;set_stat(ST_REFRESH);}
	if (s==60) {s=0; set_stat(ST_ALCHG); m++;}
	if (m==60) {m=0; h++;}
	if (h==24) {h=0;}

	if (blink_ms<blink_max) blink_ms++;
	else {set_stat(ST_REFRESH); blink^=1;blink_ms=0;}

	//UART modul krsek idtllpshez
	if (esp_timeout) esp_timeout--;
	
	//breszts idtllpse
	if (beep_ms<100) beep_ms++;
	else if (al_timeout) {al_timeout--; beep_ms=0;}
	else clear_stat(ST_ALARM);

	//visszaszmll
	if (cd_en) cd_ms++;
	if (cd_ms==100)
	{
		cd_ms=0;

		if (cd_s) 
		{
			cd_s--;
		}
		else if (cd_m)
		{
			cd_s = 59;
			cd_m--;
		}
		else if (cd_h)
		{
			cd_m=59;
			cd_h--;
		}
	}

	if (cd_en && !cd_m && !cd_h && !cd_s)
	{
		alarm_start();
		cd_en = 0;
	}

	//pesgsmentests
	if (prell_bcnt) prell_bcnt--;

	//universal, for timeing purposes
	if (dcntr) dcntr--;
}

//button pressed or released
ISR(INT5_vect)
{
	//felfut l detektls
	//gomb felengeds
	if (prell_btn && btn_is(ROT_C))
	{
		//akci vgrehajtsa idtartam szerint
		if (prell_bcnt<(PRELL_TOUT-PRELL_G2)) event_set(EVENT_BTN2);
		else if(prell_bcnt<(PRELL_TOUT-PRELL_G1)) event_set(EVENT_BTN1);

		prell_btn = 0;
		prell_bcnt = 0;
		return;
	}

	//lefut l detektls
	//gomb lenyoms
	if ((!prell_btn) && (!(btn_is(ROT_C))))
	{
		prell_btn = 1;				//gomb lenyomva
		prell_bcnt = PRELL_TOUT;	//szmll indtsa
	}
}

//rot b change
ISR(INT6_vect)
{
	if (btn_is(ROT_B))
	{
		if (btn_is(ROT_A)){event_set(EVENT_RCW); event_clr(EVENT_RCCW);}
		else {event_set(EVENT_RCCW); event_clr(EVENT_RCW);}
	}
	else
	{
		if (btn_is(ROT_A)) {event_set(EVENT_RCCW); event_clr(EVENT_RCW);}
		else {event_set(EVENT_RCW); event_clr(EVENT_RCCW);}
	}
}

ISR(USART1_RX_vect)
{
	uint8_t d = UDR1;

	if (state_start == st_mach)
	{
		if(pgm_read_byte(str_get+cnt_get)==d) cnt_get++;
		else if(pgm_read_byte(str_get+cnt_post)==d) cnt_post++;
		
		//enter mode if necessary
		if (2 == cnt_get) {cnt_get = 0; st_mach = state_get;}
		else if (3 == cnt_post) {cnt_post = 0; st_mach = state_post;}
	}

	//set data

	//send data
	if (state_get)
	{

	}
	
}
