/*
RENDSZER SZINT FGGVNYEK
ESEMNYKEZELS, BOOTLOADER HVS, RENDSZERVLTOZK
AZ ESZKZZEL S A FIRMWARE-REL KAPCSOLATOS KONSTANSOK
*/

#ifndef SYS_H
#define SYS_H

#include "sysio.h"
#include <avr/interrupt.h>

//definitions
#define DSP_DP		2		//dot mask

#define PRELL_TOUT	200		//prell timeout (*10ms)
#define PRELL_G1	4		//gesture 1 time (*10ms)
#define PRELL_G2	25		//gesture 2 time, more than g1

#define ESP_READY	1		//ready-t kldtt a modul
#define ESP_OK		2		//OK-t kldtt a modul
#define ESP_ERROR	4		//ERROR-t kldtt a modul
#define ESP_PACK	8		//csomag rkezett

#define ESP_CON_OFF	0
#define ESP_CON_AP	1		//configure as access point
#define ESP_CON_CL	2		//configure as client

#define AL_TIME		2		//seconds for the alarm
#define BLINK_TIME	60		//blink toggle time

//charset
extern const char character[23];
extern const char html_begin[];
extern const char html_end[];
extern const char str_get[4];
extern const char str_post[5];
extern const char str_el[5];

extern volatile uint8_t st_mach;	//state of state machine
extern volatile uint8_t cnt_get;	//GET string compare match counter
extern volatile uint8_t cnt_post;	//POST string compare match counter
extern volatile uint8_t cnt_el;		//counter for empty line compare match
enum HTML_state{state_start, state_get, state_post, state_el, state_read, state_send};

//system status codes
#define ST_REFRESH	1		//refresh screen
#define ST_ALARM	2		//alarm is active
#define ST_ESP_CR	4		// \r char arrived on UART
#define ST_ALCHG	8		//check alarms

//system global variables
extern volatile uint8_t h, m, s, ms;	//hour, minute, second, milliseconds
extern volatile uint8_t blink;			//for blinking dots and beep
extern volatile uint8_t status;			//status info
extern volatile uint8_t mode;			//time, countdown, alarm, setup
extern volatile uint8_t al_stat;		//LSB 4bit, mellyik breszts van engedlyezve
extern volatile uint8_t al_day[4];		//melyik napokon bresszen
extern volatile uint8_t al_hour[4];		//milyen rban bresszen
extern volatile uint8_t al_min[4];		//melyik percben bresszen
extern volatile uint8_t cd_h, cd_m, cd_s;	//visszaszmll vltozk
extern volatile uint8_t cd_en, cd_ms;		//visszaszmll engedlyezs
extern volatile uint8_t al_timeout;		//counter foe alarm off event
extern volatile uint8_t al_def_time;	//specifies the beeping time (sec)
extern volatile uint8_t beep_ms;		//for counting 10ms for better timing
extern volatile uint8_t blink_ms;		//ms for blink speed
extern volatile uint8_t blink_max;		//toggle time
extern volatile uint8_t dcntr;			//univeresal downcounter

//for wifi comm

extern volatile uint8_t esp_timeout;
extern volatile uint8_t esp_llen;

extern char str_ssid[32];
extern char str_wifipass[32];

//inline functions to manage status bits
#define is_stat(a) (status&a)
#define set_stat(a) (status|=a)
#define clear_stat(a) (status&=~a)

//higher level functions
void alarm_start();

//DEVICE INFO
extern const uint8_t bootpass[6];

uint8_t bootloader_pwd();
uint8_t bootloader_check();

//esp8266 module interface functions
void esp_conf(uint8_t _con);

uint8_t esp_ack_wait(uint8_t type, uint8_t tout);

void esp_read_ack();

//initialize all variables to default values
void init_vars();

//send string over wifi, substitute % with v as 
void uart_send_par(uint8_t v);

#endif