/*
A RENDSZER KI S BEMENETEIRT FELELS
A KI S BEMENETI JELEKET ITT VLASZTJUK EL, REJTJK FGGVNYHVSOK MG
HOGY MAGT A RENDSZERT A HERDVERTL FGGETLENL, MAGYAS PROGRAMOZSI SZINTEN
S JELENTSEN KEVESEBB ELVI HIBVAL LEHESSEN MEGRNI
*/

#ifndef SYSIO_H
#define SYSIO_H

#include <inttypes.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

//ROTARY ENCODER INPUTS, PORTD
#define ROT_A		32		//rot encoder a
#define ROT_B		64		//rot encoder b -> INT6
#define ROT_C		16		//rot encoder button -> INT5
#define PRELL_C		100		//button disable timeout

//ALARM OUTPUT
#define ALARM_O		128		//alarm out, PORTC 7

//DISPLAY OUTPUTS
#define DSP_SCLK	2		//PORTB	1
#define DSP_RCK		128		//PORTD 7
#define DSP_CLR		64		//PORTB 6
#define DSP_EN		128		//PORTB 7
#define DSP_MOSI	4		//PORTB 2

#define DSP_BRG		72		//default brightness

//ESP8266 MODULE, UART INTERFACE
#define ESP_EN		4		//enable wifi, PORTC 2
#define ESP_RST		2		//reset wifi, PORTD 1
#define ESP_GPIO0	1		//select boot mode, PORTD 0

#define ESP_BUFSIZE	64		//UART buffer size

#define ESP_MODE_FW	0		//firmware update start mode
#define ESP_MODE_N	1		//normal operation mode

//EVENT DRIVEN IO
#define EVENT_BTN1		1		//button gesture 1 (less than 0.8 secs)
#define EVENT_BTN2		2		//button gesture 2 (more than 0.8 secs)
#define EVENT_RCW		4		//rotary encoder clockwise (-)
#define EVENT_RCCW		8		//rotary encoder counter clockwise (+)

//driver functions
void btn_on();		//init rotary encoder and button and interrupts
void btn_off();		//disable rotary encoder and button interrupts
#define btn_is(a)	PIND&a

void disp_on();		//init SPI for display, PWM for dimming
void disp_off();	//disable SPI, switch off the PWM
void disp_set_brg(uint8_t val);	//set brightness of display
uint8_t disp_get_brg();	//get the brightness value
void disp_show();	//send out the disp_buf, if SPI is enabled

void alarm_on();	//init alarm output, and timeout values
void alarm_off();	//disable alarm outputs
void alarm_val(uint8_t val);	//starts if true, stops if false

void clock_on();	//start timer interrupt for system timing
void clock_off();	//disable timer interrupt

void uart_on();		//init UART for esp8266 module, 250k baud rate
void uart_off();	//disable UART interrupts, halt the module
void uart_send(const char * c_str);	//send a NULL terminated char string
void uart_send_eeprom(const char * e_c_str);	//send a NULL termintaed string from EEPROM from given address
void esp_on(uint8_t mode);	//start esp8266 module in the given mode
void esp_off();				//halt the module

//OTHER GLOBAL VARIABLES
//for display, not used in interrupt
extern uint8_t disp_buf[6];
extern volatile uint8_t event_flag;
extern volatile uint8_t esp_stat;		//MSB - power, LSB 3:0 state
extern volatile uint8_t esp_buffer[ESP_BUFSIZE];
extern volatile uint8_t esp_bufptr;

//for anti prell purposes
extern volatile uint8_t prell_btn;
extern volatile uint8_t prell_bcnt;

//macros for event handling
#define event_set(a)	(event_flag|=(a))
#define event_clr(a)	(event_flag&=~(a))
#define event_is(a)		(event_flag&(a))

#endif