#ifndef USB_H
#define USB_H

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stdint.h>

uint8_t buffer[64];	// read write buffer
uint8_t	SU;			// data are ready to send by usart..

#define SREG _SFR_IO8(0x3F)

#define STR_MANUFACTURER		L"HEXAGON"
#define STR_PRODUCT				L"ALARMCLOCK"
#define VENDOR_ID				0x16C0
#define PRODUCT_ID				0x0480
#define RAWHID_USAGE_PAGE		0xFFAB	// recommended: 0xFF00 to 0xFFFF
#define RAWHID_USAGE			0x0200	// recommended: 0x0100 to 0xFFFF
#define RAWHID_TX_SIZE			64	// transmit packet size
#define RAWHID_TX_INTERVAL		2	// max # of ms between transmit packets
#define RAWHID_RX_SIZE			64	// receive packet size
#define RAWHID_RX_INTERVAL		8	// max # of ms between receive packets

#define EP_TYPE_CONTROL				0x00
#define EP_TYPE_BULK_IN				0x81
#define EP_TYPE_BULK_OUT			0x80
#define EP_TYPE_INTERRUPT_IN		0xC1
#define EP_TYPE_INTERRUPT_OUT		0xC0
#define EP_TYPE_ISOCHRONOUS_IN		0x41
#define EP_TYPE_ISOCHRONOUS_OUT		0x40

#define EP_SINGLE_BUFFER			0x02
#define EP_DOUBLE_BUFFER			0x06

#define ENDPOINT0_SIZE		64
#define RAWHID_INTERFACE	0
#define RAWHID_TX_ENDPOINT	1
#define RAWHID_RX_ENDPOINT	2
#define RAWHID_TX_BUFFER	EP_SINGLE_BUFFER//EP_DOUBLE_BUFFER
#define RAWHID_RX_BUFFER	EP_SINGLE_BUFFER//EP_DOUBLE_BUFFER


#define EP_SIZE(s)	((s) > 32 ? 0x30 :	\
((s) > 16 ? 0x20 :	\
((s) > 8  ? 0x10 :	\
0x00)))

#define PLL_CONFIG() (PLLCSR = ((1<<PLLE)|(1<<PLLP0)))
#define USB_CONFIG() (USBCON = (1<<USBE))
#define USB_FREEZE() (USBCON = ((1<<USBE)|(1<<FRZCLK)))

#define MAX_ENDPOINT		4

#define LSB(n) (n & 255)
#define MSB(n) ((n >> 8) & 255)

// standard control endpoint request types
#define GET_STATUS			0
#define CLEAR_FEATURE			1
#define SET_FEATURE			3
#define SET_ADDRESS			5
#define GET_DESCRIPTOR			6
#define GET_CONFIGURATION		8
#define SET_CONFIGURATION		9
#define GET_INTERFACE			10
#define SET_INTERFACE			11
// HID (human interface device)
#define HID_GET_REPORT			1
#define HID_GET_IDLE			2
#define HID_GET_PROTOCOL		3
#define HID_SET_REPORT			9
#define HID_SET_IDLE			10
#define HID_SET_PROTOCOL		11

void usb_init();					// initialize everything
uint8_t usb_configured(void);		// is the USB port configured
int8_t usb_recv(uint8_t *buffer, uint8_t timeout);  // receive a packet, with timeout
int8_t usb_send(const uint8_t *buffer, uint8_t timeout); // send a packet, with timeout


#endif // ifndef USB_H
